<?php
defined('ADMIN_WS_API_PATH') or die('Hacking attempt!');

include_once(PHPWG_ROOT_PATH.'include/common.inc.php');
include_once(PHPWG_ROOT_PATH.'include/ws_core.inc.php'); 

function admin_validate_group_add($name)
{
  if (empty($name))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'The name of a group must not contain " or \' or be empty.');
  }

	// check if the group already exists
  $query = '
SELECT COUNT(*)
  FROM '.GROUPS_TABLE.'
  WHERE name = \''.pwg_db_real_escape_string($name).'\'
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count != 0)
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'This name is already used by another group.');
    }

    return TRUE;
}

function admin_group_add($name)
{
    // creating the group
    $query = '
INSERT INTO '.GROUPS_TABLE.'
  (name)
  VALUES
  (\''.pwg_db_real_escape_string($name).'\')
;';

  pwg_query($query);

      $query = '
SELECT *
  FROM '.GROUPS_TABLE.'
  WHERE name = \''.$name.'\'
;';
      $row = pwg_db_fetch_assoc(pwg_query($query));

      return new PwgNamedArray($row, 'item');
}

function admin_validate_group_delete($group_id)
{
  if (empty($group_id))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'Group_id is missing. Please enter the group_id.');
  }

  return TRUE;
}

function admin_group_delete($group_id)
{
	// destruction of the access linked to the group
      $query = '
    DELETE
      FROM '.GROUP_ACCESS_TABLE.'
      WHERE group_id = '.$group_id.'
    ;';
      pwg_query($query);
      
      // destruction of the users links for this group
      $query = '
    DELETE
      FROM '.USER_GROUP_TABLE.'
      WHERE group_id = '.$group_id.'
    ;';
      pwg_query($query);
    
      $query = '
    SELECT name
      FROM '.GROUPS_TABLE.'
      WHERE id = '.$group_id.'
    ;';
      list($groupname) = pwg_db_fetch_row(pwg_query($query));
      
      // destruction of the group
      $query = '
    DELETE
      FROM '.GROUPS_TABLE.'
      WHERE id = '.$group_id.'
    ;';
      pwg_query($query);

    $output = array("message" => sprintf(l10n('group "%s" deleted'), $groupname));
    return new PwgNamedArray($output, 'item');
}

function admin_validate_group_rename($group_id, $name)
{
  if (empty($group_id))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'Group_id is missing. Please enter the group_id.');
  }

  if (empty($name))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'The name of a group must not contain " or \' or be empty.');
  }	

	// ensure the group exists
  $query = '
SELECT COUNT(*)
  FROM '.GROUPS_TABLE.'
  WHERE id = '.$group_id.'
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count == 0)
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'This group does not exist.');
    }

    return TRUE;
}

function admin_group_rename($group_id, $name)
{
    $query = '
        UPDATE '.GROUPS_TABLE.'
        SET name = \''.pwg_db_real_escape_string($name).'\'
        WHERE id = '.$group_id.'
      ;';
    pwg_query($query);

    $output = array("message" => sprintf(l10n('group renamed to "%s"'), $name));
    return new PwgNamedArray($output, 'item');
}

function admin_validate_categories_permissions_addGroup($group_id, $cat_ids)
{
  if (empty($cat_ids))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'Cat_ids is missing. Please enter a least one cat_id.');
  }

  if (empty($group_id))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'Group_id is missing. Please enter the group_id.');
  }

	// ensure ALL the given categories exists
  $query = '
SELECT COUNT(*)
  FROM '.CATEGORIES_TABLE.'
  WHERE id IN ('.implode(',', $cat_ids).')
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count != count($cat_ids))
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'At least one category in cat_ids does not exist.');
    }

	// ensure the group exists
  $query = '
SELECT COUNT(*)
  FROM '.GROUPS_TABLE.'
  WHERE id = '.$group_id.'
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count == 0)
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'This group does not exist.');
    }

    return TRUE;
}

function admin_categories_permissions_addGroup($group_id, $cat_ids)
{
  include_once(PHPWG_ROOT_PATH.'admin/include/functions.php');

  $uppercats = get_uppercat_ids($cat_ids);
  $private_uppercats = array();

  $query = '
SELECT id
  FROM '.CATEGORIES_TABLE.'
  WHERE id IN ('.implode(',', $uppercats).')
  AND status = \'private\'
;';
  $result = pwg_query($query);
  while ($row = pwg_db_fetch_assoc($result))
  {
    array_push($private_uppercats, $row['id']);
  }

  // retrying to authorize a category which is already authorized may cause
  // an error (in SQL statement), so we need to know which categories are
  // accesible
  $authorized_ids = array();

  $query = '
SELECT cat_id
  FROM '.GROUP_ACCESS_TABLE.'
  WHERE group_id = '.$group_id.'
;';
  $result = pwg_query($query);

  while ($row = pwg_db_fetch_assoc($result))
  {
    array_push($authorized_ids, $row['cat_id']);
  }

  $inserts = array();
  $to_autorize_ids = array_diff($private_uppercats, $authorized_ids);
  foreach ($to_autorize_ids as $to_autorize_id)
  {
    array_push(
      $inserts,
      array(
        'group_id' => $group_id,
        'cat_id' => $to_autorize_id
        )
      );
  }

  mass_inserts(GROUP_ACCESS_TABLE, array('group_id','cat_id'), $inserts);

  $output = array("message" => sprintf(l10n('Access granted')));
  return new PwgNamedArray($output, 'item');
}

function admin_validate_categories_permissions_deleteGroup($group_id, $cat_ids)
{
  if (empty($cat_ids))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'Cat_ids is missing. Please enter a least one cat_id.');
  }

  if (empty($group_id))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'Group_id is missing. Please enter the group_id.');
  }

	// ensure ALL the given categories exists
  $query = '
SELECT COUNT(*)
  FROM '.CATEGORIES_TABLE.'
  WHERE id IN ('.implode(',', $cat_ids).')
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count != count($cat_ids))
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'A least one category in cat_ids does not exist.');
    }

	// ensure the group exists
  $query = '
SELECT COUNT(*)
  FROM '.GROUPS_TABLE.'
  WHERE id = '.$group_id.'
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count == 0)
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'This group does not exist.');
    }

    return TRUE;
}

function admin_categories_permissions_deleteGroup($group_id, $cat_ids)
{
  include_once(PHPWG_ROOT_PATH.'admin/include/functions.php');

  // if you forbid access to a category, all sub-categories become
  // automatically forbidden
  $subcats = get_subcat_ids($cat_ids);
  $query = '
DELETE
  FROM '.GROUP_ACCESS_TABLE.'
  WHERE group_id = '.$group_id.'
  AND cat_id IN ('.implode(',', $subcats).')
;';
  pwg_query($query);

  $output = array("message" => sprintf(l10n('Access removed')));
  return new PwgNamedArray($output, 'item');
}

function admin_validate_user_add($username, $password, $email)
{
  if(empty($username))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'Username is missing. Please enter the username.');
  }
  else if(empty($password))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'Password is missing. Please enter the password.');
  }
  else if(empty($email))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'Email address is missing. Please specify an email address.');
  }

  return TRUE;
}

function admin_user_add($username, $password, $email)
{
  global $conf;
  define('IN_ADMIN', true);
  include_once(PHPWG_ROOT_PATH.'admin/include/functions.php');

  // if (empty($params['pwg_token']) or get_pwg_token() != $params['pwg_token'])
  // {
  //   return new PwgError(403, 'Invalid security token');
  // }

    $registerUserErrors = register_user($username, $password, $email, false);

    if (count($registerUserErrors) == 0)
    {
      $query = '
SELECT *
  FROM '.USERS_TABLE.'
  WHERE '.$conf['user_fields']['username'].' = \''.pwg_db_real_escape_string($username).'\'
;';
      $row = pwg_db_fetch_assoc(pwg_query($query));

      return new PwgNamedArray($row, 'item');
    }
    else
    {
      return new PwgError(403, $registerUserErrors[0]);
    }
}

function admin_validate_user_updateDetails($username, $password, $email)
{
  global $conf;

  if (empty($username))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'Username is missing. Please enter the username.');
  }

  if (empty($password) && empty($email))
  {
    return new PwgError(WS_ERR_INVALID_PARAM, 'No update data supplied.  Please enter a password, email address or both.');
  }


  $query = '
SELECT *
  FROM '.USERS_TABLE.'
  WHERE '.$conf['user_fields']['username'].' = \''.pwg_db_real_escape_string($username).'\'
;';
  $userdata = pwg_db_fetch_assoc(pwg_query($query));

  if (empty($userdata['id']))
  	return new PwgError(WS_ERR_INVALID_PARAM, 'Username does not exist. Please enter a valid username.');

  $userid = $userdata['id'];

  if(!empty($email))
  {
  	$mail_error = validate_mail_address($userid, $email);
    if (!empty($mail_error))
    	return new PwgError(WS_ERR_INVALID_PARAM, $mail_error);
  }

  return TRUE;
}

function admin_user_updateDetails($username, $password, $email)
{
  global $conf;
  define('IN_ADMIN', true);
  include_once(PHPWG_ROOT_PATH.'admin/include/functions.php');

  // if (empty($params['pwg_token']) or get_pwg_token() != $params['pwg_token'])
  // {
  //   return new PwgError(403, 'Invalid security token');
  // }

    $data = array();
    $data{$conf['user_fields']['username']} = $username;
    $fields = array();

    if (isset($password) && $password != "")
    {
    	array_push($fields, $conf['user_fields']['password']);
        // password is hashed with function $conf['password_hash']
        $data{$conf['user_fields']['password']} = $conf['password_hash']($password);
    }

    if (isset($email) && $email != "")
    {
      array_push($fields, $conf['user_fields']['email']);
      $data{$conf['user_fields']['email']} = $email;
    }

    mass_updates(USERS_TABLE,
                   array(
                    'primary' => array($conf['user_fields']['username']),
                    'update' => $fields
                    ),
                   array($data));

  $output = array("message" => sprintf(l10n('User updated.')));
  return new PwgNamedArray($output, 'item');
}

function admin_user_delete($username)
{
  global $conf;

  // if (empty($params['pwg_token']) or get_pwg_token() != $params['pwg_token'])
  // {
  //   return new PwgError(403, 'Invalid security token');
  // }

      $query = '
SELECT *
  FROM '.USERS_TABLE.'
  WHERE '.$conf['user_fields']['username'].' = \''.pwg_db_real_escape_string($username).'\'
;';
      $row = pwg_db_fetch_assoc(pwg_query($query));

      // destruction of the users links for this group
      $query = '
    DELETE
      FROM '.USER_GROUP_TABLE.'
      WHERE user_id = '.$row['user_id'].'
    ;';
      pwg_query($query);

      $query = '
DELETE
  FROM '.USERS_TABLE.'
  WHERE '.$conf['user_fields']['username'].' = \''.pwg_db_real_escape_string($username).'\'
;';
  $row = pwg_db_fetch_assoc(pwg_query($query));

  $output = array("message" => sprintf(l10n('User deleted.')));
  return new PwgNamedArray($output, 'item');
}

function admin_validate_groups_addUser($group_id, $user_id)
{
	$query = '
SELECT user_id
  FROM '.USER_GROUP_TABLE.'
  WHERE group_id = '.$group_id.'
  AND user_id = '.$user_id.'
;';

    $associated = array_from_query($query, 'user_id');

    if (count($associated) > 0)
    {
    	return new PwgError(WS_ERR_INVALID_PARAM, 'User is already in the group.');
    }

    return TRUE;
}

function admin_groups_addUser($group_id, $user_id)
{
	$datas = array(); 

    array_push($datas,
               array('group_id'=>$group_id,
                     'user_id'=>$user_id));

    mass_inserts(USER_GROUP_TABLE,
               array('group_id', 'user_id'),
               $datas);

  $output = array("message" => sprintf(l10n('User added to group.')));
  return new PwgNamedArray($output, 'item');
}

function admin_validate_group_deleteUser($group_id, $user_id)
{
	$query = '
SELECT user_id
  FROM '.USER_GROUP_TABLE.'
  WHERE group_id = '.$group_id.'
  AND user_id = '.$user_id.'
;';

    $associated = array_from_query($query, 'user_id');

    if (count($associated) == 0)
    {
    	return new PwgError(WS_ERR_INVALID_PARAM, 'User is not in the group.');
    }

    return TRUE;
}

function admin_group_deleteUser($group_id, $user_id)
{
	$query = '
DELETE FROM '.USER_GROUP_TABLE.'
  WHERE group_id = '.$group_id.'
  AND user_id = '.$user_id.'
';
    pwg_query($query); 

	$output = array("message" => sprintf(l10n('User removed from group.')));
	return new PwgNamedArray($output, 'item');
}

function admin_validate_icy_acls_groups_addAlbum($groupname, $cat_id)
{
	$conf_path = PHPWG_ROOT_PATH.PWG_LOCAL_DIR.'config/icy_acl.zml';
	if (!file_exists($conf_path)) {
    	return new PwgError(WS_ERR_INVALID_PARAM, 'config/icy_acl.zml does nost exist. Ensure the icy_picture_modify plugin is installed and configured.');
	}

	if (strpos($groupname, ' ') !== FALSE)
		return new PwgError(WS_ERR_INVALID_PARAM, 'The icy_picture_modify plugin can not handle Group names with spaces.');

	// check if the group exists
  $query = '
SELECT COUNT(*)
  FROM '.GROUPS_TABLE.'
  WHERE name = \''.pwg_db_real_escape_string($groupname).'\'
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count == 0)
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'This group does not exist.');
    }

	// check if the album exists
  $query = '
SELECT COUNT(*)
  FROM '.CATEGORIES_TABLE.'
  WHERE id = '.pwg_db_real_escape_string($cat_id).'
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count == 0)
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'This album (cat_id) does not exist.');
    }

    return TRUE;
}

function admin_icy_acls_groups_addAlbum($groupname, $cat_id)
{
  $conf_path = PHPWG_ROOT_PATH.PWG_LOCAL_DIR.'config/icy_acl.zml';

  $zml = "
## Added by web service: icy.acls.groups.addAlbum.
$groupname:
  edit_image_of: $cat_id, sub
  create_gallery_to: $cat_id, sub
  upload_image_to: $cat_id, sub
  replace_image_of: $cat_id, sub
  delete_image_of: $cat_id, sub
  present_image_to: $cat_id, sub
";

  file_put_contents($conf_path, $zml, FILE_APPEND);
}

function admin_validate_icy_acls_groups_addUser($username, $groupname)
{
	$conf_path = PHPWG_ROOT_PATH.PWG_LOCAL_DIR.'config/icy_acl.zml';
	if (!file_exists($conf_path)) {
    	return new PwgError(WS_ERR_INVALID_PARAM, 'config/icy_acl.zml does nost exist. Ensure the icy_picture_modify plugin is installed and configured.');
	}

	// check if the user exists
  $query = '
SELECT COUNT(*)
  FROM '.USERS_TABLE.'
  WHERE username = \''.pwg_db_real_escape_string($username).'\'
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count == 0)
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'This user does not exist.');
    }

	// check if the group exists
  $query = '
SELECT COUNT(*)
  FROM '.GROUPS_TABLE.'
  WHERE name = \''.pwg_db_real_escape_string($groupname).'\'
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count == 0)
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'This group does not exist.');
    }

    return TRUE;
}

function admin_icy_acls_groups_addUser($username, $groupname)
{
  $conf_path = PHPWG_ROOT_PATH.PWG_LOCAL_DIR.'config/icy_acl.zml';

  $zml = "
## Added by web service: icy.acls.groups.addUser.
$username: @$groupname
";

  file_put_contents($conf_path, $zml, FILE_APPEND);
}

function admin_validate_icy_acls_groups_deleteUser($username, $groupname)
{
	$conf_path = PHPWG_ROOT_PATH.PWG_LOCAL_DIR.'config/icy_acl.zml';
	if (!file_exists($conf_path)) {
    	return new PwgError(WS_ERR_INVALID_PARAM, 'config/icy_acl.zml does nost exist. Ensure the icy_picture_modify plugin is installed and configured.');
	}

	// check if the user exists
  $query = '
SELECT COUNT(*)
  FROM '.USERS_TABLE.'
  WHERE username = \''.pwg_db_real_escape_string($username).'\'
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count == 0)
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'This user does not exist.');
    }

	// check if the group exists
  $query = '
SELECT COUNT(*)
  FROM '.GROUPS_TABLE.'
  WHERE name = \''.pwg_db_real_escape_string($groupname).'\'
;';
    list($count) = pwg_db_fetch_row(pwg_query($query));
    if ($count == 0)
    {
      return new PwgError(WS_ERR_INVALID_PARAM, 'This group does not exist.');
    }

    return TRUE;
}

function admin_icy_acls_groups_deleteUser($username, $groupname)
{
  $conf_path = PHPWG_ROOT_PATH.PWG_LOCAL_DIR.'config/icy_acl.zml';

  $zml = "
## Added by web service: icy.acls.groups.addUser.
$username: @$groupname
";

	$contents = file_get_contents($conf_path);
	$contents = str_replace($zml, '', $contents);
	file_put_contents($conf_path, $contents);
}

?>